aboutsummaryrefslogtreecommitdiff
path: root/src
diff options
context:
space:
mode:
authorzeertzjq <zeertzjq@outlook.com>2021-09-19 13:13:44 +0800
committerzeertzjq <zeertzjq@outlook.com>2021-09-19 13:13:44 +0800
commit963474321bcc02a5698c40b4677cd36ab8d4cebc (patch)
tree5f066f3b2899c08756d29395f2a4b699f34be1f9 /src
parent924e8e4f2d88ee5c45e521e9f758b7c9f247a011 (diff)
downloadrneovim-963474321bcc02a5698c40b4677cd36ab8d4cebc.tar.gz
rneovim-963474321bcc02a5698c40b4677cd36ab8d4cebc.tar.bz2
rneovim-963474321bcc02a5698c40b4677cd36ab8d4cebc.zip
vim-patch:8.1.1071: cannot get composing characters from the screen
Problem: Cannot get composing characters from the screen. Solution: Add screenchars() and screenstring(). (partly by Ozaki Kiichi, closes vim/vim#4059) https://github.com/vim/vim/commit/2912abb3a2fd72074e3901c8ae1d4a77ce764675
Diffstat (limited to 'src')
-rw-r--r--src/nvim/eval.lua2
-rw-r--r--src/nvim/eval/funcs.c65
-rw-r--r--src/nvim/testdir/test_utf8.vim35
-rw-r--r--src/nvim/testdir/view_util.vim19
4 files changed, 104 insertions, 17 deletions
diff --git a/src/nvim/eval.lua b/src/nvim/eval.lua
index 5f355abe3c..c0a18b3236 100644
--- a/src/nvim/eval.lua
+++ b/src/nvim/eval.lua
@@ -290,9 +290,11 @@ return {
rubyeval={args=1},
screenattr={args=2},
screenchar={args=2},
+ screenchars={args=2},
screencol={},
screenpos={args=3},
screenrow={},
+ screenstring={args=2},
search={args={1, 4}},
searchcount={args={0,1}},
searchdecl={args={1, 3}},
diff --git a/src/nvim/testdir/test_utf8.vim b/src/nvim/testdir/test_utf8.vim
index c51fb3a759..acf15e0e28 100644
--- a/src/nvim/testdir/test_utf8.vim
+++ b/src/nvim/testdir/test_utf8.vim
@@ -1,5 +1,6 @@
" Tests for Unicode manipulations
+source view_util.vim
" Visual block Insert adjusts for multi-byte char
func Test_visual_block_insert()
@@ -61,6 +62,40 @@ func Test_getvcol()
call assert_equal(2, virtcol("']"))
endfunc
+func Test_screenchar_utf8()
+ new
+
+ " 1-cell, with composing characters
+ call setline(1, ["ABC\u0308"])
+ redraw
+ call assert_equal([0x0041], screenchars(1, 1))
+ call assert_equal([0x0042], screenchars(1, 2))
+ call assert_equal([0x0043, 0x0308], screenchars(1, 3))
+ call assert_equal("A", screenstring(1, 1))
+ call assert_equal("B", screenstring(1, 2))
+ call assert_equal("C\u0308", screenstring(1, 3))
+
+ " 2-cells, with composing characters
+ let text = "\u3042\u3044\u3046\u3099"
+ call setline(1, text)
+ redraw
+ call assert_equal([0x3042], screenchars(1, 1))
+ call assert_equal([0], screenchars(1, 2))
+ call assert_equal([0x3044], screenchars(1, 3))
+ call assert_equal([0], screenchars(1, 4))
+ call assert_equal([0x3046, 0x3099], screenchars(1, 5))
+
+ call assert_equal("\u3042", screenstring(1, 1))
+ call assert_equal("", screenstring(1, 2))
+ call assert_equal("\u3044", screenstring(1, 3))
+ call assert_equal("", screenstring(1, 4))
+ call assert_equal("\u3046\u3099", screenstring(1, 5))
+
+ call assert_equal([text . ' '], ScreenLinesUtf8(1, 8))
+
+ bwipe!
+endfunc
+
func Test_list2str_str2list_utf8()
" One Unicode codepoint
let s = "\u3042\u3044"
diff --git a/src/nvim/testdir/view_util.vim b/src/nvim/testdir/view_util.vim
index 1def201a05..dd143c02c1 100644
--- a/src/nvim/testdir/view_util.vim
+++ b/src/nvim/testdir/view_util.vim
@@ -34,6 +34,25 @@ function! ScreenLines(lnum, width) abort
return lines
endfunction
+" Get text on the screen, including composing characters.
+" ScreenLines(lnum, width) or
+" ScreenLines([start, end], width)
+function! ScreenLinesUtf8(lnum, width) abort
+ redraw!
+ if type(a:lnum) == v:t_list
+ let start = a:lnum[0]
+ let end = a:lnum[1]
+ else
+ let start = a:lnum
+ let end = a:lnum
+ endif
+ let lines = []
+ for l in range(start, end)
+ let lines += [join(map(range(1, a:width), 'screenstring(l, v:val)'), '')]
+ endfor
+ return lines
+endfunction
+
function! ScreenAttrs(lnum, width) abort
redraw!
if type(a:lnum) == v:t_list
diff --git a/src/nvim/eval/funcs.c b/src/nvim/eval/funcs.c
index 35ebd0f3f8..a71e33dada 100644
--- a/src/nvim/eval/funcs.c
+++ b/src/nvim/eval/funcs.c
@@ -8108,9 +8108,7 @@ static void f_rpcstop(typval_T *argvars, typval_T *rettv, FunPtr fptr)
}
}
-/*
- * "screenattr()" function
- */
+// "screenattr()" function
static void f_screenattr(typval_T *argvars, typval_T *rettv, FunPtr fptr)
{
int c;
@@ -8128,9 +8126,7 @@ static void f_screenattr(typval_T *argvars, typval_T *rettv, FunPtr fptr)
rettv->vval.v_number = c;
}
-/*
- * "screenchar()" function
- */
+// "screenchar()" function
static void f_screenchar(typval_T *argvars, typval_T *rettv, FunPtr fptr)
{
int c;
@@ -8148,11 +8144,34 @@ static void f_screenchar(typval_T *argvars, typval_T *rettv, FunPtr fptr)
rettv->vval.v_number = c;
}
-/*
- * "screencol()" function
- *
- * First column is 1 to be consistent with virtcol().
- */
+// "screenchars()" function
+static void f_screenchars(typval_T *argvars, typval_T *rettv, FunPtr fptr)
+{
+ int row = tv_get_number_chk(&argvars[0], NULL) - 1;
+ int col = tv_get_number_chk(&argvars[1], NULL) - 1;
+ if (row < 0 || row >= default_grid.Rows
+ || col < 0 || col >= default_grid.Columns) {
+ tv_list_alloc_ret(rettv, 0);
+ return;
+ }
+ ScreenGrid *grid = &default_grid;
+ screenchar_adjust_grid(&grid, &row, &col);
+ int pcc[MAX_MCO];
+ int c = utfc_ptr2char(grid->chars[grid->line_offset[row] + col], pcc);
+ int composing_len = 0;
+ while (pcc[composing_len] != 0) {
+ composing_len++;
+ }
+ tv_list_alloc_ret(rettv, composing_len + 1);
+ tv_list_append_number(rettv->vval.v_list, c);
+ for (int i = 0; i < composing_len; i++) {
+ tv_list_append_number(rettv->vval.v_list, pcc[i]);
+ }
+}
+
+// "screencol()" function
+//
+// First column is 1 to be consistent with virtcol().
static void f_screencol(typval_T *argvars, typval_T *rettv, FunPtr fptr)
{
rettv->vval.v_number = ui_current_col() + 1;
@@ -8184,17 +8203,29 @@ static void f_screenpos(typval_T *argvars, typval_T *rettv, FunPtr fptr)
tv_dict_add_nr(dict, S_LEN("endcol"), ecol);
}
-/*
- * "screenrow()" function
- */
+// "screenrow()" function
static void f_screenrow(typval_T *argvars, typval_T *rettv, FunPtr fptr)
{
rettv->vval.v_number = ui_current_row() + 1;
}
-/*
- * "search()" function
- */
+// "screenstring()" function
+static void f_screenstring(typval_T *argvars, typval_T *rettv, FunPtr fptr)
+{
+ rettv->vval.v_string = NULL;
+ rettv->v_type = VAR_STRING;
+ int row = tv_get_number_chk(&argvars[0], NULL) - 1;
+ int col = tv_get_number_chk(&argvars[1], NULL) - 1;
+ if (row < 0 || row >= default_grid.Rows
+ || col < 0 || col >= default_grid.Columns) {
+ return;
+ }
+ ScreenGrid *grid = &default_grid;
+ screenchar_adjust_grid(&grid, &row, &col);
+ rettv->vval.v_string = vim_strsave(grid->chars[grid->line_offset[row] + col]);
+}
+
+// "search()" function
static void f_search(typval_T *argvars, typval_T *rettv, FunPtr fptr)
{
int flags = 0;